#!/usr/bin/env python
# -*- coding: utf-8 -*-
if __name__ == '__main__':
    #try:
    #    import psyco
    #    psyco.full()
    #except ImportError:
    #    pass
    from dynts import runtests
    runtests()