<div class="econometric-plot">
<div class="data" style="display:none">{% if item %}
<span class="item">{{ item }}</span>{% endif %}
<span class="height">{{ height }}</span>
<a href="{{ url }}"></a>
</div>
</div>